Package: aqp
Version: 2.3.1
Title: Algorithms for Quantitative Pedology
Authors@R: c(person(given="Dylan", family="Beaudette", role = c("aut", "cre"), email = "dylan.beaudette@usda.gov", comment = c(ORCID="0009-0008-2780-4785")), 
             person(given="Pierre", family="Roudier", email="roudierp@landcareresearch.co.nz", role = c("aut", "ctb")), 
             person(given="Andrew", family="Brown", email="andrew.g.brown@usda.gov", role = c("aut", "ctb")), 
             person(given="Stephen", family="Roecker", role = c("aut", "ctb")),
             person(given = "David", family = "Rossiter", role = c("ctb")))
Author: Dylan Beaudette [aut, cre] (ORCID:
    <https://orcid.org/0009-0008-2780-4785>),
  Pierre Roudier [aut, ctb],
  Andrew Brown [aut, ctb],
  Stephen Roecker [aut, ctb],
  David Rossiter [ctb]
Maintainer: Dylan Beaudette <dylan.beaudette@usda.gov>
Depends: R (>= 4.1.0)
Imports: grDevices, graphics, stats, utils, methods, grid, lattice,
        cluster, data.table, farver, digest, colorspace, ape
Suggests: mvtnorm, soilDB, sp, sf, latticeExtra, tactile, compositions,
        markovchain, xtable, testthat, Gmedian, Hmisc, tibble,
        RColorBrewer, scales, mpspline2 (>= 0.1.9), soiltexture, gower,
        knitr, rmarkdown, dendextend
Description: The Algorithms for Quantitative Pedology (AQP) project was started in 2009 to organize a loosely-related set of concepts and source code on the topic of soil profile visualization, aggregation, and classification into this package (aqp). Over the past 8 years, the project has grown into a suite of related R packages that enhance and simplify the quantitative analysis of soil profile data. Central to the AQP project is a new vocabulary of specialized functions and data structures that can accommodate the inherent complexity of soil profile information; freeing the scientist to focus on ideas rather than boilerplate data processing tasks <doi:10.1016/j.cageo.2012.10.020>. These functions and data structures have been extensively tested and documented, applied to projects involving hundreds of thousands of soil profiles, and deeply integrated into widely used tools such as SoilWeb <https://casoilresource.lawr.ucdavis.edu/soilweb-apps>. Components of the AQP project (aqp, soilDB, sharpshootR, soilReports packages) serve an important role in routine data analysis within the USDA-NRCS Soil Science Division. The AQP suite of R packages offer a convenient platform for bridging the gap between pedometric theory and practice.
License: GPL (>= 3)
LazyLoad: yes
Repository: CRAN
URL: https://ncss-tech.github.io/aqp/, https://ncss-tech.github.io/AQP/
BugReports: https://github.com/ncss-tech/aqp/issues
Language: en-US
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/Needs/website: rmarkdown
NeedsCompilation: no
Packaged: 2026-03-19 17:12:25 UTC; Dylan.Beaudette
Date/Publication: 2026-03-20 06:11:09 UTC
