# Generated by roxygen2: do not edit by hand

S3method(.DollarNames,SoilProfileCollection)
export(HzDepthLogicSubset)
export(L1_profiles)
export(NCSP)
export(ReactionClassLevels)
export(SANN_1D)
export(SoilProfileCollection)
export(SoilTextureLevels)
export(accumulateDepths)
export(addBracket)
export(addDiagnosticBracket)
export(addVolumeFraction)
export(aggregateColor)
export(aggregateColorPlot)
export(aggregateSoilDepth)
export(alignTransect)
export(allocate)
export(aqp.env)
export(argillic.clay.increase.depth)
export(barron.torrent.redness.LAB)
export(bootstrapSoilTexture)
export(brierScore)
export(buntley.westin.index)
export(checkHzDepthLogic)
export(checkSPC)
export(col2Munsell)
export(collapseHz)
export(colorChart)
export(colorContrast)
export(colorContrastPlot)
export(colorQuantiles)
export(colorVariation)
export(compareSites)
export(compositeSPC)
export(confusionIndex)
export(contrastChart)
export(contrastClass)
export(correctAWC)
export(crit.clay.argillic)
export(denormalize)
export(depthOf)
export(depthWeights)
export(dissolve_hz)
export(duplicate)
export(electroStatics_1D)
export(equivalentMunsellChips)
export(estimateAWC)
export(estimatePSCS)
export(estimateSoilColor)
export(estimateSoilDepth)
export(evalGenHZ)
export(evalMissingData)
export(explainPlotSPC)
export(fillHzGaps)
export(findOverlap)
export(fixOverlap)
export(flagOverlappingHz)
export(formatMunsell)
export(fragmentClasses)
export(fragmentSieve)
export(fragvol_to_texmod)
export(genSlabLabels)
export(generalize.hz)
export(genhzTableToAdjMat)
export(get.increase.depths)
export(get.increase.matrix)
export(get.ml.hz)
export(get.slice)
export(getArgillicBounds)
export(getCambicBounds)
export(getClosestMunsellChip)
export(getLastHorizonID)
export(getMineralSoilSurfaceDepth)
export(getPlowLayerDepth)
export(getSoilDepthClass)
export(getSurfaceHorizonDepth)
export(glom)
export(glomApply)
export(grepSPC)
export(groupSPC)
export(groupedProfilePlot)
export(guessGenHzLevels)
export(guessHzAttrName)
export(guessHzDesgnName)
export(guessHzTexClName)
export(harden.melanization)
export(harden.rubification)
export(hasDarkColors)
export(horizonColorIndices)
export(huePosition)
export(huePositionCircle)
export(hurst.redness)
export(hzAbove)
export(hzBelow)
export(hzDepthTests)
export(hzDistinctnessCodeToOffset)
export(hzOffset)
export(hzTopographyCodeToLineType)
export(hzTopographyCodeToOffset)
export(hzTransitionProbabilities)
export(hz_dissolve)
export(hz_intersect)
export(hz_lag)
export(hz_segment)
export(hz_to_taxpartsize)
export(invertLabelColor)
export(launderMunsell)
export(lookup_taxpartsize)
export(lunique)
export(maxDepthOf)
export(minDepthOf)
export(missingDataGrid)
export(mixMunsell)
export(mollic.thickness.requirement)
export(mostLikelyHzSequence)
export(mu_confusion_matrix)
export(munsell2rgb)
export(mutate_profile)
export(mutate_profile_raw)
export(overlapMetrics)
export(panel.depth_function)
export(parseMunsell)
export(pbindlist)
export(perturb)
export(ph_to_rxnclass)
export(plotColorMixture)
export(plotColorQuantiles)
export(plotMultipleSPC)
export(plotProfileDendrogram)
export(plotSPC)
export(plot_distance_graph)
export(prepanel.depth_function)
export(previewColors)
export(profileGroupLabels)
export(profileInformationIndex)
export(quickSPC)
export(random_profile)
export(rebuildSPC)
export(reduceSPC)
export(repairMissingHzDepths)
export(rgb2munsell)
export(rp)
export(rxnclass_to_ph)
export(segment)
export(shannonEntropy)
export(shuffle)
export(sim)
export(simulateColor)
export(slab_function)
export(slice.fast)
export(slicedHSD)
export(soilColorSignature)
export(soilPalette)
export(soilTextureColorPal)
export(spc_in_sync)
export(spec2Munsell)
export(splitLogicErrors)
export(ssc_to_texcl)
export(subApply)
export(summarizeSPC)
export(summaryTauW)
export(tauW)
export(texcl_to_ssc)
export(texmod_to_fragvoltot)
export(textureTriangleSummary)
export(texture_to_taxpartsize)
export(texture_to_texmod)
export(thicknessOf)
export(thompson.bell.darkness)
export(validateMunsell)
export(warpHorizons)
export(xtableTauW)
exportClasses(SoilProfileCollection)
exportMethods("GHL<-")
exportMethods("aqp_df_class<-")
exportMethods("coordinates<-")
exportMethods("depth_units<-")
exportMethods("depths<-")
exportMethods("diagnostic_hz<-")
exportMethods("horizonDepths<-")
exportMethods("horizonNames<-")
exportMethods("horizons<-")
exportMethods("hzID<-")
exportMethods("hzdesgnname<-")
exportMethods("hzidname<-")
exportMethods("hzmetaname<-")
exportMethods("hztexclname<-")
exportMethods("initSpatial<-")
exportMethods("metadata<-")
exportMethods("prj<-")
exportMethods("profile_id<-")
exportMethods("proj4string<-")
exportMethods("replaceHorizons<-")
exportMethods("restrictions<-")
exportMethods("site<-")
exportMethods("siteNames<-")
exportMethods(GHL)
exportMethods(aqp_df_class)
exportMethods(as.character)
exportMethods(as.data.frame)
exportMethods(c)
exportMethods(combine)
exportMethods(coordinates)
exportMethods(depth_units)
exportMethods(depths)
exportMethods(diagnostic_hz)
exportMethods(dice)
exportMethods(generalizeHz)
exportMethods(getSpatial)
exportMethods(harmonize)
exportMethods(horizonDepths)
exportMethods(horizonNames)
exportMethods(horizons)
exportMethods(hzDesgn)
exportMethods(hzID)
exportMethods(hzMetadata)
exportMethods(hzdesgnname)
exportMethods(hzidname)
exportMethods(hzmetaname)
exportMethods(hztexclname)
exportMethods(idname)
exportMethods(isEmpty)
exportMethods(length)
exportMethods(max)
exportMethods(metadata)
exportMethods(min)
exportMethods(munsell2spc)
exportMethods(names)
exportMethods(nrow)
exportMethods(plot)
exportMethods(prj)
exportMethods(profileApply)
exportMethods(profile_id)
exportMethods(proj4string)
exportMethods(reorderHorizons)
exportMethods(restrictions)
exportMethods(site)
exportMethods(siteNames)
exportMethods(slab)
exportMethods(slice)
exportMethods(spc2mpspline)
exportMethods(split)
exportMethods(subset)
exportMethods(subsetHz)
exportMethods(subsetProfiles)
exportMethods(transform)
exportMethods(trunc)
exportMethods(unique)
exportMethods(validSpatialData)
import(data.table)
importFrom(ape,as.phylo)
importFrom(ape,plot.phylo)
importFrom(ape,tiplabels)
importFrom(cluster,daisy)
importFrom(cluster,pam)
importFrom(cluster,silhouette)
importFrom(colorspace,deutan)
importFrom(colorspace,protan)
importFrom(colorspace,tritan)
importFrom(digest,digest)
importFrom(farver,compare_colour)
importFrom(farver,convert_colour)
importFrom(grDevices,chull)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,colors)
importFrom(grDevices,convertColor)
importFrom(grDevices,grey)
importFrom(grDevices,hsv)
importFrom(grDevices,rgb)
importFrom(grDevices,rgb2hsv)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,grconvertX)
importFrom(graphics,grid)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(grid,gpar)
importFrom(grid,grid.text)
importFrom(grid,unit)
importFrom(lattice,levelplot)
importFrom(lattice,panel.abline)
importFrom(lattice,panel.grid)
importFrom(lattice,panel.lines)
importFrom(lattice,panel.points)
importFrom(lattice,panel.polygon)
importFrom(lattice,panel.rect)
importFrom(lattice,panel.segments)
importFrom(lattice,panel.text)
importFrom(lattice,strip.custom)
importFrom(lattice,trellis.par.get)
importFrom(lattice,xyplot)
importFrom(methods,"slot<-")
importFrom(methods,.hasSlot)
importFrom(methods,as)
importFrom(methods,isGeneric)
importFrom(methods,new)
importFrom(methods,prototype)
importFrom(methods,representation)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,setOldClass)
importFrom(methods,setReplaceMethod)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(stats,TukeyHSD)
importFrom(stats,aggregate)
importFrom(stats,aov)
importFrom(stats,approxfun)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,as.hclust)
importFrom(stats,cmdscale)
importFrom(stats,complete.cases)
importFrom(stats,cov)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,splinefun)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,weighted.mean)
importFrom(stats,xtabs)
importFrom(utils,.DollarNames)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,object.size)
importFrom(utils,packageVersion)
importFrom(utils,str)
